var Cufflinks = function()
{
	var cufflinks = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	var rawBioAssays;
	
	// Page initialization
	cufflinks.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', cufflinks.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Cufflinks.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmed';
		Wizard.asyncJsonRequest(url, cufflinks.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	cufflinks.initializeStep1 = function(response)
	{
		rawBioAssays = response.rawBioAssays;
		if (rawBioAssays.length == 0)
		{
			Wizard.setFatalError('No raw bioassays available for processing.');
			return;
		}
		
		var html = '<table id="rawBioAssaysTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="6">Reads</th>';
		html += '<th class="dottedleft" colspan="3">Cufflinks</th>'
		html += '<th class="dottedleft" colspan="3">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Library</th>';
		html += '<th class="dottedleft">Plate</th>';
		html += '<th class="dottedleft">PF_READS</th>';
		html += '<th>PT_READS</th>';
		html += '<th>PM_READS</th>';
		html += '<th>ALIGNED_PAIRS</th>';
		html += '<th>Duplicates</th>'
		html += '<th class="warning-col"></th>';
		html += '<th class="dottedleft">Job</th>'
		html += '<th>FPKM</th>';
		html += '<th>Files</th>';
		html += '<th class="dottedleft">Flag RNA</th>';
		html += '<th>Re-run</th>';
		html += '<th>Gene report</th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		var numWithError = 0;
		var yellowImg = '<img src="../images/yellow-label.png">';
		for (var rawNo = 0; rawNo < rawBioAssays.length; rawNo++)
		{
			var raw = rawBioAssays[rawNo];

			var lib = raw.lib;
			var aligned = raw.aligned;
			var masked = raw.masked;
			var merged = raw.merged;

			lib.stratagene = Reggie.isStratagene(lib.name);
			lib.external = Reggie.isExternal(lib.name);
			var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			
			var pfReads = merged.PF_READS == null ? '-' : Reggie.formatNumber(merged.PF_READS/1000000, null, 2)
			var ptReads = merged.PT_READS == null ? '-' : Reggie.formatNumber(merged.PT_READS/1000000, null, 2)
			var maskedReads = masked.PM_READS == null ? '-' : Reggie.formatNumber(masked.PM_READS/1000000, null, 2)
			var alignedReads = aligned.ALIGNED_PAIRS == null ? '-' : Reggie.formatNumber(aligned.ALIGNED_PAIRS/1000000, null, 2);
			var ptPercent = merged.PF_READS && merged.PT_READS ? ' (' + Math.round(100*merged.PT_READS/merged.PF_READS) + '%)' : '';
			var alignedPercent = aligned.ALIGNED_PAIRS && merged.PF_READS ? ' (' + Math.round(100*aligned.ALIGNED_PAIRS/merged.PF_READS) + '%)' : '';
			var percentDuplicates = aligned.FRACTION_DUPLICATION == null ? '-' : Math.round(100*aligned.FRACTION_DUPLICATION)+'%';

			var cufflinksJob = raw.cufflinksJob;
			var limits = cufflinksJob.limits;
			var isError = false;
			var flagChecked = false;
	
			html += '<tr class="highlight ' + (rawNo % 4 < 2 ? 'evenrow' : 'oddrow') + (isYellow ? ' yellow-specimen' : '') + '">';
			html += '<td class="prompt if-yellow">'+img+Strings.encodeTags(raw.name)+'</td>';
			html += '<td class="dottedleft">'+(libPlate ? Strings.encodeTags(libPlate.name) : '-') +'</td>';

			html += '<td class="dottedleft">'+pfReads+'</td>';
			html += '<td>'+ptReads+ptPercent+'</td>';
			html += '<td>'+maskedReads+'</td>';
			html += '<td>'+alignedReads+alignedPercent+'</td>';
			html += '<td>'+percentDuplicates+'</td>';
			html += '<td class="warning-col">';
			if (aligned.ALIGNED_PAIRS == null || aligned.ALIGNED_PAIRS < limits.minAlignedPairsFlag)
			{
				html += '<img src="../images/warning.png" title="Less than '+Reggie.formatCount(limits.minAlignedPairsFlag) + ' aligned pairs">';
				flagChecked = true;
			}
			html += '</td>';

			html += '<td class="dottedleft">';
			if (cufflinksJob)
			{
				isError |= cufflinksJob.status == 'ERROR';
				var image = JOB_IMAGE[cufflinksJob.status];
				if (!isError && !raw.fpkmFile) image = 'warning.png';
				html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+cufflinksJob.id+'" title="'+Strings.encodeTags(cufflinksJob.statusMessage)+'"><img src="../images/'+image+'"></span>';
			}
			html += '</td>';
			html += '<td>';
			html += raw.fpkmFile ? 'Yes' : 'No';
			html += '</td>';

			if (raw.DataFilesFolder && cufflinksJob && cufflinksJob.server)
			{
				html += '<td>';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(cufflinksJob.server)+'" data-folder="'+Strings.encodeTags(raw.DataFilesFolder)+'" data-item-id="'+aligned.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td>-</td>';
			}
			// Do not allow flagging external or stratagene
			var geneReportChecked = raw.fpkmFile && !isError ? ' checked' : '';
			var reRunChecked = isError ? ' checked' : '';
			var geneReportDisabled = isError || !raw.fpkmFile ? ' disabled' : '';
			if (isError) numWithError++;
			
			if (lib.stratagene || lib.external)
			{
				html += '<td class="dottedleft">-</td>';
			}
			else
			{
				html += '<td class="dottedleft"><input type="checkbox" name="flag.'+raw.id+'"'+(flagChecked ? ' checked':'')+'></td>';
			}
			
			html += '<td><input type="checkbox" name="rerun.'+raw.id+'"'+reRunChecked+'></td>';
			html += '<td><input type="checkbox" name="geneReport.'+raw.id+'"'+geneReportChecked+geneReportDisabled+'></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+raw.id+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('rawBioAssays').innerHTML = html;
		if (numWithError > 0) Doc.show('delete-items', 'table');
		
		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', cufflinks.fileLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	cufflinks.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = 'view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	
	cufflinks.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.rawBioAssays = all;
		for (var rawNo = 0; rawNo < rawBioAssays.length; rawNo++)
		{
			var raw = rawBioAssays[rawNo];
			var tmp = {};
			tmp.id = raw.id;
			
			tmp.flag = frm['flag.'+raw.id] && frm['flag.'+raw.id].checked;
			tmp.rerun = frm['rerun.'+raw.id].checked;
			tmp.geneReport = frm['geneReport.'+raw.id].checked;
			tmp.comment = frm['comment.'+raw.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../Cufflinks.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterCufflinks';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, cufflinks.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	cufflinks.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return cufflinks;
}();

Doc.onLoad(Cufflinks.initPage);

